
package diagrams;

import java.awt.geom.AffineTransform;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import static java.lang.Math.atan2;
import static java.lang.Math.sin;
import static java.lang.Math.cos;
import static java.lang.Math.min;
import static java.lang.Math.sqrt;
import static java.lang.Math.toDegrees;
import static java.lang.Math.toRadians;
import static java.lang.Math.round;
import java.util.Enumeration;


/**
 *
 * @author B. Gaertner
 */
public class PolarGrid extends AbstractGrid {

    private LinearScale linearScale;
    private PolarGridProperties angularRangeScale;
    private int width;
    private int height;
    private SimplePaintRange paintRange;
    private ScaleTransformation scaleTransformation;

    public PolarGrid (LinearScale ls, PolarGridProperties as, int w, int h ) {
        linearScale = ls;
        angularRangeScale = as;
        width = w;
        height = h;
        paintRange = new SimplePaintRange(0, min(width/2, height/2) - angularRangeScale.inset);
        scaleTransformation = this.linearScale.createTransformation(paintRange);
    }

    @Override
    protected void renderGridOn(Graphics2D g) {
        int xCenter = this.width / 2;
        int yCenter = this.height/ 2;

        g.setFont(this.linearScale.scaleProps.scaleFont);
        FontMetrics fm = g.getFontMetrics(this.linearScale.scaleProps.scaleFont);
        ValueFormatter fmt = this.linearScale.scaleProps.formatter;

        for (Enumeration<TickValue> e = linearScale.enumeration(); e.hasMoreElements();) {
            TickValue tv = e.nextElement();
            float radius = (float)scaleTransformation.transform(tv.value);
            int iRadius = round(radius);
            g.drawOval(xCenter - iRadius, yCenter - iRadius, 2*iRadius, 2*iRadius);
        }

        int tickDistance = this.angularRangeScale.tickDistance;
        int labelDistance = this.angularRangeScale.labelDistance;
        int upperLimit = (int)(360.0/tickDistance + 0.0);
        int div = tickDistance;

        for (int i = 0; i < upperLimit; i++) {
            int angle = i*div;
            double s = sin(toRadians(angle));
            double c = cos(toRadians(angle));
            int lb = angle % 90 == 0
                      ? 0
                      : (angle % 30 == 0)
                        ? 1
                        : angle % 10 == 0 ? 2 : 4;

            double p0 = this.scaleTransformation.transform(this.linearScale.valueAtTick(lb));
            double p1 = this.scaleTransformation.transform(this.linearScale.valueAtTick(this.linearScale.scaleRange.numberOfIntervals));
            p1 -= p1/500;  //  dieser Trick vermeidet ueberschiessende Strichenden am aeusseren Kreis
            if (labelDistance == 1 || i % labelDistance == 0) {
                p1 += 6;
            }
            // print("p0 = " + str(p0) + " p1 = " + str(p1))
            int x1 = (int)(round(c*p1 + xCenter));  // outer circle
            int y1 = (int)(round(yCenter - s*p1));
            int x0 = (int)(round(c*p0 + xCenter));  // inner circle
            int y0 = (int)(round(yCenter - s*p0));
            g.drawLine(x0, y0, x1, y1);
            if (labelDistance == 1 || i % labelDistance == 0) {
                AffineTransform currentTransformation = g.getTransform();
                g.translate(xCenter, yCenter);
                g.rotate(toRadians(-i*div));
                g.translate(paintRange.paintEndpoint() + 10, 0);
                g.rotate(toRadians(90));
                String labelString = fmt.format(i*div);
                int w = fm.stringWidth(labelString);
                g.drawString(labelString, -w/2, -4);
                g.setTransform(currentTransformation);
            }
        }
    }

    @Override
    public int[] transform(double phi, double rho) {
        int[] res = new int[2];
        this.transform(phi, rho, res);
        return res;
    }

    @Override
    public void transform(double phi, double rho, int[] res) {
        int xCenter = this.width / 2;
        int yCenter = this.height/ 2;
        double radius = scaleTransformation.transform(rho);
        double x = radius*cos(phi);
        double y = radius*sin(phi);

        res[0] = round((float)(xCenter + x));
        res[1] = round((float)(yCenter - y));
    }

    @Override
    public void transformViewCoordinates(int x, int y, PositionRecorder p) {
        int xCenter = this.width / 2;
        int yCenter = this.height/ 2;
        int radius = this.paintRange.paintWidth();
        int deltaX = x - xCenter;
        int deltaY = y - yCenter;
        double distance = sqrt(deltaX*deltaX + deltaY*deltaY);
        if (distance <= radius) {
            double angle = toDegrees(atan2(deltaY, -deltaX)) + 180.0;
            double r2 = linearScale.createTransformation(paintRange).inverseTransformation(distance);
            p.setPosition(r2, angle);
        } else {
            p.clearPosition();
        }

    }
}
